'if Err then msgbox "Cam engine failed: " & Err.Description
'...:....1....:....2....:....3....:....4....:....5....:....6....:....7....:....8
' WilliamsCams - routines specific to Williams games
' version 3.
'
' This file contains all constants, routines and functions necessary to
' Williams machines.
'
' WillamsCams routines Copyright (c) 2006 Star Wolf Innovations.
' All Rights Reserved.
' Author: Greywolf <greywolf@starSPAMwolfSTOP.com>
' The author releases this material to be redistributed only on a
' not-for-profit basis.  Credit given for routines used from
' this code collection would be greatly appreciated.
Option Explicit
const emWilliamsCamsVersion=3.2
' Last update: 2006:0522:1430
' 3.2 initial release to be in sync with other CAMS engines.
' C_USER1 => C_RSTB for bonus-initiated reset cam-stepper routine
' C_USER2 => C_USER1 for user-defined cam-stepper routine
' Added C_RSTATE for state-reset cam-stepper routine
' And away we go.
' **** Constants, with variables close by.
' Limits
const	GLOB_MAXBALLS=10	' no more than this
const	GLOB_MAXPLAYERS=4	' no more than this
const	GLOB_MAXBONUS=20	' Williams topped out at 20


const	NPULSES=5		' Williams repeaters have 6 pulses
const GLOB_MAXREPLAYS=25

' Scoring

public	score_min		' minimum score
		score_min = 10		' currently unused

' Bonus
dim	bonus_style			' default: 0 (fast collect)
dim bonus_mult_style	' default: 1 (slow collect)\

' Bonus Mult Style, explained:
' If the bonus to be collected is slow, the first two or three
' steps are points and the 4th one is the bonus stepper;
' if it is fast, bonus is stepped down every third step
' and two or three points are collected every half-cycle.

const	BONUS_FAST=0		' fast (5/cycle) collect
const	BONUS_SLOW=1		' slow (2/cycle) collect

bonus_style = BONUS_FAST		' point/step x 5
bonus_mult_style = BONUS_SLOW	' point-point-(point/pause)-pause-step

' APPLIES TO SEQUENTIAL BONUS ONLY!!!
dim	collecting_bonus
const	COLL_NONE=0		' not currently collecting bonus
const	COLL_RESET=1	' decrement but do not score (always superfast)
const	COLL_DECR=2		' collecting and decrementing
const	COLL_KEEP=3		' collecting and keeping (doesn't happen on
						' Williams tables that I know...

dim		reset_spsa		' reset SPSA
		reset_spsa=0	' set to 1 after the ball is loaded

' What the backbox says...
const	SHOOT_AGAIN="SAME PLAYER SHOOTS AGAIN"
' Gottlieb says "SHOOT AGAIN".
' Williams says either/or.
'****
' Cam State
' The following ONLY apply to values resulting from a cam start, such as
' the lit specials on the rollovers on flip-flop, or the standing 5000
' target on Wizard.

const	C_SMASK=&h0000000f'	MUST FIGURE OUT HOW TO REDUCE SCORING!
const	SCORE_R1=	&H0001	' reel 1 score
const	SCORE_R2=	&H0002	' reel 2 score
const	SCORE_R3=	&H0004	' reel 3 score
const	SCORE_R4=	&H0008	' reel 4 score
	' 	*** features ***
'* can we squeeze some bits out of the scoring?
'2.710: yes, we just did.  &H1000 is not used.
'2.710: also, since addb/decb are counters of the same
'2.710: process, we can use &H6000 as its own bit.
	' NOTE: VB likes to think that &H8000 is -32768 :/
const	C_ADDP=	&H00000010	' add a player
const	C_SPEC=	&H00000020	' score a special
const	C_REPLAY=	&H00000040	' add a credit
const	C_DREPL=	&H00000080	' take a credit
const	C_EJECT=	&H00000100	' cause a saucer to eject
const	C_ADDB=	&H00000200	' add bonus (calls add_bonus)
const	C_DECB=	&H00000400	' decrease bonus (calls dec_bonus)
const	C_RDROPS=	&H00000800	' reset drop targets
const	C_DPOST=	&H00001000	' retract the flipper post
const	C_CGATE=	&H00002000	' close an open gate (calls close_gate)
const	C_LOADB=	&H00004000	' load the ball into the shooter
const	C_RSPSA=	32768		' reset SPSA
const	C_NEXT=	&H00010000	' go to next player/ball
const	C_RES2=	&H00020000	' RESERVED
const	C_GAMEON=	&H00040000	' engage game-on relay
const	C_GAMEOFF=	&H00080000	' true end-of-game
const C_RXREEL=	&H00100000	' reset reel pulse
const C_RSTATE=	&H00200000	' reset state
const   C_RSTB=	&H00400000	' User-defined cam pulse #1 - called from collect.
const   C_USER1=	&H00800000	' User-defined cam pulse #2 - not.
const C_ADVBALL=&H01000000	' Advance the ball counter
const C_ADDBALL=&H02000000	' add a ball (decrement ball counter)...
const	C_RTILT=	&H80000000	' reset tilt
const   C_FMASK=	&HFFFFFFF0	' feature mask

tablestate.tilt_style = tablestate.tilt_style OR TILT_STEALS_SPSA

'*** CAM ENGINE
dim incam		' we in the cam engine already?
dim cp			' cam position
dim camstack(): redim camstack(NPULSES)

public u1sub

'*** SPECIFICS
dim	CurrentBank
dim	CurrentEject
dim	CurrentPost
dim	CurrentGate
dim	CurrentAltq
dim	CurrentBQueue
dim	CurrentScoring
dim	CurrentCycler
dim	CurrentBuilder

dim	Ball

camtimer.interval = TIMER_WILLIAMS
dim clight(5)

'dim x, l
'x = 0
'for each l in array(l_cam0,l_cam1,l_cam2,l_cam3,l_cam4,l_cam5,l_cam6,l_cam7)
'	set clight(x)= l
'	x = x + 1
'next

sub runcam()
	dim ao
'showcams
	if incam then exit sub
	incam = true
	cp = 0
	camtimer.enabled = true
end sub

' user-settable routines
sub crstb(p)
end sub

sub cuser1(p)
end sub

sub ce_reset_handler(p)
end sub

' to override the blank cRSTB routine...
sub ce_SetRSTB(s)
	executeglobal "sub crstb(p): " & s & " p: end sub"
end sub

sub ce_SetUser1(s)
	executeglobal "sub cuser1(p): " & s & " p: end sub"
end sub

sub ce_HandleReset(s)
	executeglobal "sub ce_reset_handler(p):" & s & " p: end sub"
end sub

sub clearcams()
	dim i
	for i = 0 to ubound(camstack)
		camstack(i) = 0
	next
end sub

sub camtimer_timer()
	dim o, r
	dim dt
	dim sr	' state reset

	addval camstack(cp)
	playsound S_camclick	' something suitably soft for a cam-hitting-relay
	me.enabled = false

' 	clight(max(0,cp - 1)).state = lightstateoff
	if (cp < ubound(camstack)) then
		dbx "Pending: " & C_pending & "; game state: " & TableState.game
		me.enabled = true
'		clight(cp).state = lightstateon
		cp = cp + 1
		incam = true
	else
		' clear possible state
		clearcams
		CurrentBank = NULL
		CurrentGate = NULL
		playsound S_coinlock
		incam = false
		emAltQChange ALTQ_CAM
		' check for state here, in this order:
		' 1.  Drop targets
		' 2.  Eject holes
		' 3.  Bonus
		' 4.  Drain
		' NOTE:  This should probably be more of a
		' evaldrops routine to see if we should even
		' reset them.
		' ** 1.  Drop targets
		' ** XXX Williams revisit.
		dt = emCheckDrops
		if (not isEmpty(dt)) then
			CurrentBank = dt
			camstack(1) = C_RDROPS
			runcam
			exit sub
		end if
		' ** 2. Eject Holes
		dbx "Pending: " & C_pending & "; game state: " & TableState.game
		if (c_pending AND PEND_EJECT) then
			emScoreSaucers CurrentEject
			emPend PEND_EJECT, FALSE
			exit sub
			' insert eject handler here
		' ** 3. Bonus
		elseif (c_pending AND PEND_BONUS) then
			if (CurrentBQueue.multiplier > 1) then
				emCollect bonus_mult_style
			else
				emCollect bonus_style
			end if
		' ** 4. Drain
		elseif (c_pending AND PEND_DRAIN) then
			Drain_Hit
		elseif (c_pending AND PEND_RESET) then
			r = 0
			for o = 0 to Scores.maxplayers
				if Scores.score(o) <> 0 then r = r + 1
			next
			if (nbqueue >= 0) then
				for each o in BBonusQ
					if o.bonus then r = r + 1
				next
			end if
			if (r > 0) then
				camstack(1) = C_RXREEL
				camstack(2) = C_RXREEL
				camstack(3) = C_RXREEL
				camstack(4) = C_RXREEL
				camstack(5) = C_RXREEL
			else
				init_ball
				reset_spsa = 1
				emPend PEND_RESET, FALSE
			end if
			runcam
		end if
	end if
		dbx "Pending: " & C_pending & "; game state: " & TableState.game
end sub

' addval - the world is inside this call.

const	ADD_ON	= 1
const	ADD_OFF	= 2
const ADD_PULSE	= 3

dim addval_v: addval_v = 0

sub addval(v)
	addval_on v, ADD_PULSE
end sub

sub addval_on(v, down)
	' down: TRUE, engages the relay (play the sound)
	' 	  FALSE, disengage the relay (flip the reel)

	dim f	' feature
	dim s	' score
	dim x	' spare variable
	dim y	' whatever else

	s = (v and C_SMASK)

	f = (v and C_FMASK)

	if (v = TRUE) then dd "addval", "v = TRUE!  WTF?!?"

	if (down and ADD_OFF) then
		addval_v = addval_v and (not v)
	end if
' MUST ADD SCORE FIRST, or else score + bonus advance
' won't register!
' 2.710:
'	Must do a tilt check HERE
	if (tablestate.game = GAME_ON) and (not tablestate.tilted) then
		if (s) then
			if reset_spsa then 
				addval C_RSPSA
				reset_spsa = 0
			end if
		end if
		x = ((Scores.Score(Scores.curplayer) \ minscore) mod 10)
		y = ((Scores.score(Scores.curplayer) \ (minscore * 10) mod 10))
		if (s and SCORE_R1) then
			if (down and ADD_ON) then
				playsound S_highchime
				if (x = 9) then
					playsound S_midchime
					emAltQChange ALTQ_MED
					if (y = 9) then
						playsound S_lowchime
					end if
				end if
				if (matchpoint = minscore) then
					playsound S_matchwheel
				end if
			end if
			if (down and ADD_OFF) then
				Scores.Creel.Addvalue(minscore)
				addscore minscore
			end if
		end if
		if (s and SCORE_R2) then
			if (down and ADD_ON) then
				playsound S_midchime
				if (y = 9) then playsound S_lowchime
				if (matchpoint = minscore * 10) then
					playsound S_matchwheel
				end if
			end if
			if (down and ADD_OFF) then
				Scores.Creel.AddValue(minscore * 10)
				addscore minscore * 10
			end if
		end if
		x = ((Scores.score(Scores.curplayer) \ (minscore * 100) mod 10))
		if (s and SCORE_R3) then
			if (down and ADD_ON) then
				playsound S_lowchime
				if (matchpoint = minscore * 100) then
					playsound S_matchwheel
				end if
			end if
			if (down and ADD_OFF) then
				Scores.Creel.AddValue(minscore * 100)
				addscore minscore * 100
			end if
		end if
		if (s and SCORE_R4) then
			if (down and ADD_ON) then
				playsound S_altlowchime
				if (matchpoint = minscore * 1000) then
					playsound S_matchwheel
				end if
			end if
			if (down and ADD_OFF) then
				Scores.Creel.AddValue(minscore * 1000)
				addscore minscore * 1000
			end if
		end if
	end if


	'  **** FEATURES ****
	if (down and ADD_ON) then
		if (f and C_ADDP) then emAddPlayer

		if (f and C_REPLAY) then emAddReplay

		if (f and C_DREPL) then Scores.take_replay

		if (f and C_SPEC) then scores.score_Special

		if (f and C_EJECT) then emClearSaucers	CurrentEject
' note: C_RTILT = C_ADDB or C_DECB.  must check vs. mask.
		if (f and C_ADDB) then
			emAddBonus CurrentBQueue, (Scores.didscore <> 0)
		end if

		if (f and C_DECB) then emTakeBonus(CurrentBQueue)

		if (f and C_RDROPS) then emResetDrops CurrentBank

		if (f and C_DPOST) then emSetPost false

		if (f and C_CGATE) then emCloseGate CurrentGate

		if (f and C_LOADB) then
			emLoadBall()
		end if

		if (f and C_RSPSA) then TableState.shootagain false

		if (f and C_NEXT) then emNextUp

		if (f and C_RXREEL) then
			for each x in Bscore
				playsound S_camclick
				playsound S_camclick
				playsound S_camclick
				x.reel(0).timerenabled = true
				x.reset
			next
			if (Scores.maxplayers = 1) then
				if (Tablestate.ballsleft > nballs) then
					Tablestate.setball nballs
					playsound S_bstep
				elseif (Tablestate.ballsleft <> nballs) then
					Tablestate.setball Tablestate.ballsleft + 1
					playsound S_bstep
				end if
			end if
		end if

		if (f and C_GAMEOFF) then
			TableState.gameover 1
			if (Scores.maxplayers > 2) then
				playsound S_bstep
			end if
		end if

		if (f and C_ADDBALL) then
			emNextBall TRUE
		end if

		if (f and C_RSTB) then crstb cp

		if (f and C_USER1) then cuser1 cp
		
		if (f and C_RSTATE) then ce_reset_handler cp

		if (f and C_ADVBALL) then
			playsound S_bstep
			emNextBall FALSE
		end if

		if (f and C_GAMEON) then
			TableState.game = GAME_ON
			emTiltReset
		end if

		if (f and C_RTILT) then emTiltReset

	end if

	if (down and ADD_ON) then
		addval_v = addval_v or v
	end if

end sub

sub add_nx (n, x, force)
	dim pp		  ' position pointer

	if (n AND C_SMASK) AND _
		(TableState.tilted OR (TableState.game <> GAME_ON)) then
			exit sub
	end if

	if (incam) and not force then exit sub

	for pp = 1 to 5
		if (x > 0) then
			camstack(pp) = camstack(pp) or n
		end if
		x = x - 1
	next

	runcam
end sub

' add_nxe:  Add the requested value (n) x number of times at the END
' of the cycle.	 Williams liked to do this a lot.

sub add_nxe (n, x, force)
	dim pp

	if (n AND C_SMASK) AND _
		(TableState.tilted OR (TableState.game <> GAME_ON)) then
			exit sub
	end if

	if (incam) and not force then exit sub

	for pp = 5 to 1 step -1
		if (x > 0) then
			camstack(pp) = camstack(pp) or n
		end if
		x = x - 1
	next

	runcam
end sub

' some simple routines:
'

sub adv3 (n)
	add_nx n or C_ADDB, 3, true
end sub

sub add3 (n)
	add_nx n, 3, true
end sub

sub add3b (n)
	camstack(4) = C_ADDB
	add_nx n, 3, true
end sub

sub add5 (n)
	add_nx n, 5, true
end sub

sub add5b (n)
	camstack(3) = C_ADDB
	add_nx n, 5, true
end sub

' bonus collection
' Collecting in the drain or other retractor...
' Handle array collection separately.  To my knowledge,
' the only table that does array collection is OXO.

Public sub emCollect (how)
	dim x, flags

	if nbqueue < 0 then exit sub

	if (isNull(CurrentBQueue)) then exit sub
	if (tablestate.tilted) or (tablestate.game < GAME_ON) then
		x = 0
	else
		x = CurrentBQueue.worth
	end if
' if we're keeping it, pass it off to the emBScore routine
	if (collecting_bonus = COLL_KEEP) then
		emBScore CurrentBQueue, _
		((tablestate.game > GAME_OVER) and (tablestate.tilted = false))
		exit sub
	end if
	camstack(1) = C_RSTB
	camstack(2) = C_RSTB
	camstack(3) = C_RSTB
	camstack(4) = C_RSTB
	camstack(5) = C_RSTB
' NOTE:  Need to implement 5x bonus...
	if (CurrentBQueue.bonus > 0 AND scores.didscore) OR (tablestate.game > GAME_OVER) then
		emPend PEND_BONUS, TRUE
		if (how = BONUS_SLOW) then
			if (CurrentBQueue.multiplier = 1) then
				camstack(1) = camstack(1) or x
				camstack(2) = camstack(2) or C_DECB
				if (CurrentBQueue.bonus > 1) then
					camstack(3) = camstack(3) or x
					camstack(5) = camstack(5) or C_DECB
				end if
			elseif (CurrentBQueue.multiplier >= 2) then
				camstack(4) = camstack(4) or C_DECB
				camstack(1) = camstack(1) or x
				camstack(2) = camstack(2) or x
				if (CurrentBQueue.multiplier = 3) then
					camstack(3) = camstack(3) or x
				end if
			end if
		else
			if (CurrentBQueue.multiplier = 1) then
				if (CurrentBQueue.bonus >= 1) then
					camstack(1) = camstack(1) or x or C_DECB
				end if
				if (CurrentBQueue.bonus >= 2) then
					camstack(2) = camstack(2) or x or C_DECB
				end if
				if (CurrentBQueue.bonus >= 3) then
					camstack(3) = camstack(3) or x or C_DECB
				end if
				if (CurrentBQueue.bonus >= 4) then
					camstack(4) = camstack(4) or x or C_DECB
				end if
				if (CurrentBQueue.bonus >= 5) then
					camstack(5) = camstack(5) or x or C_DECB
				end if
			elseif (CurrentBQueue.multiplier > 1) then
				camstack(0) = camstack(0) or x
				camstack(1) = camstack(1) or x
				camstack(2) = camstack(2) or C_DECB
				if (CurrentBQueue.bonus > 1) then
					camstack(3) = camstack(3) or x
					camstack(4) = camstack(4) or x
					camstack(5) = camstack(5) or C_DECB
				end if
				select case CurrentBQueue.multiplier
				case 3
					camstack(2) = camstack(2) or camstack(1)
					camstack(5) = camstack(5) or camstack(4)
				end select
			end if
		end if
	else
		if TableState.game = GAME_TRANS then
			camstack(5) = C_GAMEON
		end if
		collecting_bonus = COLL_NONE
	end if
	emPend PEND_BONUS, FALSE
	runcam
end sub

'  and simply scoring on the playfield (no retracting)

dim btmp
sub emBScore(bonusq, doscore)
	dim x

	if nbqueue < 0 then exit sub
	if (isNull(bonusq)) then exit sub	' we can't collect both on field.

	collecting_bonus = COLL_KEEP
	if incam then exit sub: end if

	x = bonusq.worth

	if (btmp > 0) then
		emPend PEND_BONUS, TRUE
		if (btmp = 3) or (btmp = 4) then
			add_nxe x, btmp, true
		else
			add_nx x, btmp, true
		end if
		btmp = btmp - min(5, btmp)
	end if

	if (btmp = 0) then
		emPend PEND_EJECT, FALSE
		camstack(5) = camstack(5) OR C_EJECT
		emPend PEND_BONUS, FALSE
		collecting_bonus = COLL_NONE
	end if
	runcam
end sub


'** game init **

' init the game - Williams-style
sub emGameReset()
	dim i

	emPend PEND_RESET, TRUE

	if (Scores.reel(0).timerinterval <> (int(camtimer.interval * 0.9))) then
		Scores.reel(0).timerinterval = camtimer.interval * 0.9
	end if

	CurrentBank = NULL
	CurrentGate = NULL

	camstack(1) = camstack(1) or C_RXREEL
	camstack(2) = C_RXREEL
	camstack(3) = C_RXREEL
	camstack(4) = C_RXREEL
	camstack(5) = C_RXREEL

	TableState.setcanplay 0
	TableState.nplayers = 0
	TableState.showmatch false
	TableState.gameover false	' only sets gamestate to TRANS
	if (Scores.maxplayers <> 1) then
		PlaySound S_breset
		TableState.setball nballs
	end if
	emResetDrops NULL
	emCloseGate NULL
	emSetPost FALSE
	emBonusMX NULL, 1
	emBuilderReset CurrentBuilder, TRUE
	emCyclerSet NULL, TRUE
	TableState.setplayer 0
	emTiltReset
	'playsound S_breset
	runcam
end sub

sub emEndGame
	dim s, k
	PlaySound S_bstep
	if (tablestate.nplayers = 2) then
		k = 3
	else
		k = 2
	end if
	for s = 0 to TableState.nplayers
		if TableState.checkmatch(Scores.score(s)) then
			if (cp >= s+k) then
				addval C_REPLAY	' too late to push on stack.
			else
				camstack(s+k) = (camstack(s+k) OR C_REPLAY)
			end if
		end if
	next
	TableState.showmatch true
	TableState.tilt false
	addval C_GAMEOFF
	TableState.game = GAME_TRANS
	camstack(5) = camstack(5) or C_GAMEOFF
'	TableState.setball 0
	emBuilderReset CurrentBuilder, FALSE
	emCyclerSet NULL, FALSE
	emFLights FALSE
	runcam
	game_end	' must be provided by table
end sub

' addplayer.  KINDA Williams-specific.

sub emAddPlayer()
	if (TableState.ballsleft <> nballs) or (TableState.playerup <> 0) then
		emGameReset
		exit sub
	end if
	if TableState.addplayer then
		playsound S_addplayer
	end if
end sub

function StartButton(n)
	StartButton = false
	if (n = FALSE) then
		if (emTakeReplay() = FALSE) then
			exit function
		end if
	end if
	if TableState.nplayers >= Scores.maxplayers AND _
		TableState.ballsleft = nballs then exit function
	camstack(0) = camstack(0) or C_DREPL
	camstack(0) = camstack(0) or C_ADDP
	if Scores.maxplayers = 1 then TableState.shift_thru = TRUE
	runcam
	StartButton = true
end function

sub emNextBall(inc)	' FOR SINGLE-PLAYER TABLES
	if (inc) then
		TableState.addball true
	else
		if (TableState.IsLastBall AND Scores.didscore) then
			TableState.showmatch TRUE
			TableState.addball FALSE
			addval C_GAMEOFF
			exit sub
		end if
		TableState.addball false
	end if
end sub

dim e_
sub emNextUp ()
	dim nextp: nextp = 0
	dim shift

	if (NOT(Scores.didscore)) then
		exit sub
	end if

	CurrentGate = NULL : CurrentBank = NULL

	camstack(0) = camstack(0) or C_ADDB
	camstack(1) = camstack(1) or C_DPOST
	camstack(4) = camstack(4) or C_RTILT

	if (TableState.same_player) then
		Scores.didscore = FALSE: exit sub
	end if

	shift = (Scores.Maxplayers = 1)

	if (cp = 2) then
		nextp = TableState.nextplayer(shift)
		e_=""
	else
		if (shift) then nextp = TableState.nextplayer(shift): end if
	end if

	'e_=e_&"cp "&cp&" shift "&shift&" nextp "&nextp&vbNewLine
	if (nextp = -1) then
'		ndbx x_dbx, e_, TRUE
		clearcams
		addval C_DPOST or C_CGATE
		TableState.setplayer -1	'clear all playerup lights
		TableState.game = GAME_TRANS
		camstack(5) = camstack(5) or C_GAMEOFF
		TableState.advb shift	' ball advance
		emEndGame			' handles the match routine
		exit sub
	end if

	camstack(3) = camstack(3) or C_NEXT

	if (cp = 2) then	' 4p ball shift, 2p player shift
		if (shift) then	' 2p...
			if (tablestate.nplayers = 0) then ' 1 playing
				e_=e_&"nextp on 1 (1/2)"
				TableState.advp shift		' shift player
			end if
		else			' 4p...
			if (nextp) then	' next ball?
				e_=e_&"nextb on 1 (1/4)"
				TableState.advb shift		' shift ball
			end if
		end if
		e_=e_&vbNewLine
	end if

	if (cp = 3) then	' player shift
		if (shift) then
			if (nextp) then
				e_=e_&"nextb on 2 (2P)" & vbNewLine
				TableState.advb true
			end if
			e_=e_&"nextp on 2 (2P)" & vbNewLine
			TableState.advp true
		else
			e_=e_&"nextp on 2 (4P)" & vbNewLine
			TableState.advp shift
		end if
		Scores.didscore = FALSE
	end if
	Table_NextUp
'	ndbx x_dbx, e_, true
end sub

'#### HIGH SCORE ROUTINES.
dim show_hiscore, hiscore, def_hiscore, hidate
	hidate = #11/11/1979#
	'# DEFAULT HIGH SCORE IS SET IN configure_scores
	'# HIGH SCORE CONFIG IS IN configure_table


sub display_hiscore
	if not (isobject(x_hiscore)) then exit sub
	if hiscore = "" then hiscore = def_hiscore
	if (show_hiscore) then
		x_hiscore.text = FormatNumber(hiscore, 0, 0, 0, TRUE) &_
			" " & hidate
	else
		x_hiscore.text = ""
	end if
end sub

sub update_hiscore(reset, replay)
	dim n,x

'	ndbx x_dbx, "update_hiscore: cp " & cp, true
	if reset then
		dd "high score reset", "Resetting high score to " & def_hiscore
		hiscore = def_hiscore
		hidate = Date()
	else
		x = tablestate.nplayers
		for n = 0 to x
			if (scores.score(n) > hiscore) then
				hiscore = scores.score(n)
				hidate = Date()
				if (replay) then
					if (x = 2) then
						camstack(5) = camstack(5) and C_REPLAY
					else
						addval C_REPLAY
					end if
				end if
			end if
		next
	end if
	display_hiscore
end sub

sub bcSlamTilt()
	dim n
	emPend PEND_DRAIN, FALSE
	emPend PEND_BONUS, FALSE
	TableState.tilt_policy = TILT_ENDS_GAME
	'ndbx x_camdb, "emCheck(TableState.ptilt) =" & emCheck(TableState.ptilt), TRUE
	tilter.tilt TRUE
	TableState.tilt_style = TILT_ALL
	bqueue.setmx 1
	game_end
	TableState.tilt_policy = TILT_BALL_ONLY
	TableState.shootagain FALSE
	emClear tablestate.ptilt
end sub
